@@ -1327,7 +1327,7 @@ parse_previous_duplicate_name (const char *name,
 
 	g_assert (name[0] != '\0');
 	
-	*suffix = strrchr (name + 1, '.');
+	*suffix = strchr (name + 1, '.');
 	if (*suffix == NULL || (*suffix)[1] == '\0') {
 		/* no suffix */
 		*suffix = "";
@@ -1841,9 +1841,10 @@ nautilus_file_operations_copy_move (const GList *item_uris,
 		} else {
 			/* duplication */
 			target_uri = gnome_vfs_uri_ref (source_uri);
-			if (target_dir_uri == NULL) {
-				target_dir_uri = gnome_vfs_uri_ref (source_dir_uri);
+			if (target_dir_uri != NULL) {
+				gnome_vfs_uri_unref (target_dir_uri);
 			}
+			target_dir_uri = gnome_vfs_uri_ref (source_dir_uri);
 		}
 		
 		if (target_uri != NULL) {
@@ -2627,8 +2628,8 @@ nautilus_self_check_file_operations (void)
 	EEL_CHECK_STRING_RESULT (get_duplicate_name ("foo.txt", 1), "foo (copy).txt");
 	EEL_CHECK_STRING_RESULT (get_duplicate_name ("foo foo.txt", 1), "foo foo (copy).txt");
 	EEL_CHECK_STRING_RESULT (get_duplicate_name ("foo foo.txt txt", 1), "foo foo (copy).txt txt");
-	EEL_CHECK_STRING_RESULT (get_duplicate_name ("foo...txt", 1), "foo.. (copy).txt");
-	EEL_CHECK_STRING_RESULT (get_duplicate_name ("foo...", 1), "foo... (copy)");
+	EEL_CHECK_STRING_RESULT (get_duplicate_name ("foo...txt", 1), "foo (copy)...txt");
+	EEL_CHECK_STRING_RESULT (get_duplicate_name ("foo...", 1), "foo (copy)...");
 	EEL_CHECK_STRING_RESULT (get_duplicate_name ("foo. (copy)", 1), "foo. (another copy)");
 	EEL_CHECK_STRING_RESULT (get_duplicate_name ("foo (copy)", 1), "foo (another copy)");
 	EEL_CHECK_STRING_RESULT (get_duplicate_name ("foo (copy).txt", 1), "foo (another copy).txt");
