@@ -32,7 +32,6 @@
 #include "nautilus-global-preferences.h"
 #include "nautilus-icon-factory.h"
 #include "nautilus-icon-private.h"
-#include "nautilus-theme.h"
 #include <eel/eel-art-extensions.h>
 #include <eel/eel-gdk-extensions.h>
 #include <eel/eel-gdk-pixbuf-extensions.h>
@@ -49,6 +48,7 @@
 #include <libart_lgpl/art_rgb_affine.h>
 #include <libart_lgpl/art_rgb_rgba_affine.h>
 #include <libart_lgpl/art_svp_vpath.h>
+#include <librsvg/rsvg.h>
 #include <libgnome/gnome-i18n.h>
 #include <eel/eel-canvas-util.h>
 #include <atk/atkimage.h>
@@ -1139,7 +1139,7 @@ draw_stretch_handles (NautilusIconCanvasItem *item, GdkDrawable *drawable,
 
 	gc = gdk_gc_new (drawable);
 
-	knob_filename = nautilus_theme_get_image_path ("knob.png");
+	knob_filename = nautilus_pixmap_file ("knob.png");
 	knob_pixbuf = gdk_pixbuf_new_from_file (knob_filename, NULL);
 	knob_width = gdk_pixbuf_get_width (knob_pixbuf);
 	knob_height = gdk_pixbuf_get_height (knob_pixbuf);
@@ -1345,6 +1345,7 @@ real_map_pixbuf (NautilusIconCanvasItem *icon_item)
 	EelCanvas *canvas;
 	char *audio_filename;
 	GdkPixbuf *temp_pixbuf, *old_pixbuf, *audio_pixbuf;
+	double zoom;
 	
 	temp_pixbuf = icon_item->details->pixbuf;
 	canvas = EEL_CANVAS_ITEM(icon_item)->canvas;
@@ -1365,10 +1366,14 @@ real_map_pixbuf (NautilusIconCanvasItem *icon_item)
 		/* if the icon is currently being previewed, superimpose an image to indicate that */
 		/* audio is the only kind of previewing right now, so this code isn't as general as it could be */
 		if (icon_item->details->is_active) {
+			zoom = (double) gdk_pixbuf_get_width (temp_pixbuf) / NAUTILUS_ICON_SIZE_STANDARD;
 			/* Load the audio symbol. */
-			audio_filename = nautilus_pixmap_file ("audio.png");
+			audio_filename = nautilus_pixmap_file ("audio.svg");
 			if (audio_filename != NULL) {
-				audio_pixbuf = gdk_pixbuf_new_from_file (audio_filename, NULL);
+				audio_pixbuf = rsvg_pixbuf_from_file_at_zoom_with_max (audio_filename, zoom, zoom,
+																	   NAUTILUS_ICON_MAXIMUM_SIZE,
+																	   NAUTILUS_ICON_MAXIMUM_SIZE,
+																	   NULL);
 			} else {
 				audio_pixbuf = NULL;
 			}
@@ -1382,8 +1387,7 @@ real_map_pixbuf (NautilusIconCanvasItem *icon_item)
 					 gdk_pixbuf_get_width (temp_pixbuf),
 					 gdk_pixbuf_get_height(temp_pixbuf),
 					 0, 0,
-					 canvas->pixels_per_unit,
-					 canvas->pixels_per_unit,
+					 1.0, 1.0,
 					 GDK_INTERP_BILINEAR, 0xFF);
 				
 				g_object_unref (audio_pixbuf);
@@ -2042,7 +2046,7 @@ hit_test_stretch_handle (NautilusIconCanvasItem *item,
 		return FALSE;
 	}
 
-	knob_filename = nautilus_theme_get_image_path ("knob.png");
+	knob_filename = nautilus_pixmap_file ("knob.png");
 	knob_pixbuf = gdk_pixbuf_new_from_file (knob_filename, NULL);
 	knob_width = gdk_pixbuf_get_width (knob_pixbuf);
 	knob_height = gdk_pixbuf_get_height (knob_pixbuf);
@@ -2889,32 +2893,32 @@ nautilus_icon_canvas_item_class_init (NautilusIconCanvasItemClass *class)
 		object_class,
 		PROP_EDITABLE_TEXT,
 		g_param_spec_string ("editable_text",
-				     _("editable text"),
-				     _("the editable label"),
+				     "editable text",
+				     "the editable label",
 				     "", G_PARAM_READWRITE));
 
         g_object_class_install_property (
 		object_class,
 		PROP_ADDITIONAL_TEXT,
 		g_param_spec_string ("additional_text",
-				     _("additional text"),
-				     _("some more text"),
+				     "additional text",
+				     "some more text",
 				     "", G_PARAM_READWRITE));
 
         g_object_class_install_property (
 		object_class,
 		PROP_HIGHLIGHTED_FOR_SELECTION,
 		g_param_spec_boolean ("highlighted_for_selection",
-				      _("highlighted for selection"),
-				      _("whether we are highlighted for a selection"),
+				      "highlighted for selection",
+				      "whether we are highlighted for a selection",
 				      FALSE, G_PARAM_READWRITE)); 
 
         g_object_class_install_property (
 		object_class,
 		PROP_HIGHLIGHTED_AS_KEYBOARD_FOCUS,
 		g_param_spec_boolean ("highlighted_as_keyboard_focus",
-				      _("highlighted as keyboard focus"),
-				      _("whether we are highlighted to render keyboard focus"),
+				      "highlighted as keyboard focus",
+				      "whether we are highlighted to render keyboard focus",
 				      FALSE, G_PARAM_READWRITE)); 
 
 
@@ -2922,8 +2926,8 @@ nautilus_icon_canvas_item_class_init (NautilusIconCanvasItemClass *class)
 		object_class,
 		PROP_HIGHLIGHTED_FOR_DROP,
 		g_param_spec_boolean ("highlighted_for_drop",
-				      _("highlighted for drop"),
-				      _("whether we are highlighted for a D&D drop"),
+				      "highlighted for drop",
+				      "whether we are highlighted for a D&D drop",
 				      FALSE, G_PARAM_READWRITE)); 
 
 	item_class->update = nautilus_icon_canvas_item_update;
