@@ -119,6 +119,11 @@ struct FMListViewDetails {
 	char *original_name;
 };
 
+struct SelectionForeachData {
+	GList *list;
+	GtkTreeSelection *selection;
+};
+
 /*
  * The row height should be large enough to not clip emblems.
  * Computing this would be costly, so we just choose a number
@@ -126,6 +131,9 @@ struct FMListViewDetails {
  */
 #define LIST_VIEW_MINIMUM_ROW_HEIGHT	28
 
+/* We wait two seconds after row is collapsed to unload the subdirectory */
+#define COLLAPSE_TO_UNLOAD_DELAY 2000 
+
 static int                      click_policy_auto_value;
 static char *              	default_sort_order_auto_value;
 static gboolean			default_sort_reversed_auto_value;
@@ -133,15 +141,16 @@ static NautilusZoomLevel        default_zoom_level_auto_value;
 static GList *                  default_visible_columns_auto_value;
 static GList *                  default_column_order_auto_value;
 
-static GList *fm_list_view_get_selection   (FMDirectoryView   *view);
-static void   fm_list_view_set_zoom_level  (FMListView        *view,
-					    NautilusZoomLevel  new_level,
-					    gboolean           always_set_level);
-static void   fm_list_view_scale_font_size (FMListView        *view,
-					    NautilusZoomLevel  new_level);
-static void   fm_list_view_scroll_to_file  (FMListView        *view,
-					    NautilusFile      *file);
-static void   fm_list_view_iface_init      (NautilusViewIface *iface);
+static GList *fm_list_view_get_selection                   (FMDirectoryView   *view);
+static GList *fm_list_view_get_selection_for_file_transfer (FMDirectoryView   *view);
+static void   fm_list_view_set_zoom_level                  (FMListView        *view,
+							    NautilusZoomLevel  new_level,
+							    gboolean           always_set_level);
+static void   fm_list_view_scale_font_size                 (FMListView        *view,
+							    NautilusZoomLevel  new_level);
+static void   fm_list_view_scroll_to_file                  (FMListView        *view,
+							    NautilusFile      *file);
+static void   fm_list_view_iface_init                      (NautilusViewIface *iface);
 
 
 
@@ -248,8 +257,7 @@ fm_list_view_did_not_drag (FMListView *view,
 		if ((event->button == 1 || event->button == 2)
 		    && ((event->state & GDK_CONTROL_MASK) != 0 ||
 			(event->state & GDK_SHIFT_MASK) == 0)
-		    && view->details->row_selected_on_button_down
-		    && (click_policy_auto_value == NAUTILUS_CLICK_POLICY_DOUBLE)) {
+		    && view->details->row_selected_on_button_down) {
 			if (!button_event_modifies_selection (event)) {
 				gtk_tree_selection_unselect_all (selection);
 				gtk_tree_selection_select_path (selection, path);
@@ -305,35 +313,46 @@ drag_data_get_callback (GtkWidget *widget,
 }
 
 static void
-selection_foreach (GtkTreeModel *model,
-		   GtkTreePath *path,
-		   GtkTreeIter *iter,
-		   gpointer data)
+filtered_selection_foreach (GtkTreeModel *model,
+			    GtkTreePath *path,
+			    GtkTreeIter *iter,
+			    gpointer data)
 {
-	GList **list;
+	struct SelectionForeachData *selection_data;
+	GtkTreeIter parent;
+	GtkTreeIter child;
 	
-	list = (GList**)data;
-	
-	*list = g_list_prepend (*list, 
-				gtk_tree_row_reference_new (model, path));
+	selection_data = data;
+
+	/* If the parent folder is also selected, don't include this file in the
+	 * file operation, since that would copy it to the toplevel target instead
+	 * of keeping it as a child of the copied folder
+	 */
+	child = *iter;
+	while (gtk_tree_model_iter_parent (model, &parent, &child)) {
+		if (gtk_tree_selection_iter_is_selected (selection_data->selection,
+							 &parent)) {
+			return;
+		}
+		child = parent;
+	}
 	
+	selection_data->list = g_list_prepend (selection_data->list, 
+					       gtk_tree_row_reference_new (model, path));
 }
 
 static GList *
-get_selection_refs (GtkTreeView *tree_view)
+get_filtered_selection_refs (GtkTreeView *tree_view)
 {
-	GtkTreeSelection *selection;
-	GList *ref_list;
-	
-	ref_list = NULL;
-	
-	selection = gtk_tree_view_get_selection (tree_view);
-	gtk_tree_selection_selected_foreach (selection, 
-					     selection_foreach, 
-					     &ref_list);
-	ref_list = g_list_reverse (ref_list);
+	struct SelectionForeachData selection_data;
 
-	return ref_list;
+	selection_data.list = NULL;
+	selection_data.selection = gtk_tree_view_get_selection (tree_view);
+	
+	gtk_tree_selection_selected_foreach (selection_data.selection, 
+					     filtered_selection_foreach, 
+					     &selection_data);
+	return g_list_reverse (selection_data.list);
 }
 
 static void
@@ -414,7 +433,7 @@ motion_notify_callback (GtkWidget *widget,
 			stop_drag_check (view);
 			view->details->drag_started = TRUE;
 			
-			ref_list = get_selection_refs (GTK_TREE_VIEW (widget));
+			ref_list = get_filtered_selection_refs (GTK_TREE_VIEW (widget));
 			g_object_set_data_full (G_OBJECT (context),
 						"drag-info",
 						ref_list,
@@ -430,8 +449,10 @@ motion_notify_callback (GtkWidget *widget,
 				gtk_drag_set_icon_default (context);
 			}
 		}		      
+		return TRUE;
 	}
-	return TRUE;
+	
+	return FALSE;
 }
 
 static void
@@ -458,6 +479,7 @@ button_press_callback (GtkWidget *widget, GdkEventButton *event, gpointer callba
 	static gint64 last_click_time = 0;
 	static int click_count = 0;
 	int double_click_time;
+	int expander_size, horizontal_separator;
 
 	view = FM_LIST_VIEW (callback_data);
 	tree_view = GTK_TREE_VIEW (widget);
@@ -548,7 +570,12 @@ button_press_callback (GtkWidget *widget, GdkEventButton *event, gpointer callba
 			     (event->state & GDK_SHIFT_MASK) == 0)) {
 				view->details->row_selected_on_button_down = gtk_tree_selection_path_is_selected (selection, path);
 				if (view->details->row_selected_on_button_down) {
-					call_parent = FALSE;
+					gtk_widget_style_get (widget,
+						"expander-size", &expander_size,
+						"horizontal-separator", &horizontal_separator,
+						NULL);
+					call_parent = (event->x <= horizontal_separator / 2 +
+						gtk_tree_path_get_depth (path) * expander_size);
 				} else if  ((event->state & GDK_CONTROL_MASK) != 0) {
 					call_parent = FALSE;
 					gtk_tree_selection_select_path (selection, path);
@@ -611,7 +638,6 @@ button_release_callback (GtkWidget *widget,
 		stop_drag_check (view);
 		if (!view->details->drag_started) {
 			fm_list_view_did_not_drag (view, event);
-			return TRUE;
 		}
 	}
 	return FALSE;
@@ -739,6 +765,115 @@ popup_menu_callback (GtkWidget *widget, gpointer callback_data)
 	return TRUE;
 }
 
+static void
+subdirectory_done_loading_callback (NautilusDirectory *directory, FMListView *view)
+{
+	fm_list_model_subdirectory_done_loading (view->details->model, directory);
+}
+
+static void
+row_expanded_callback (GtkTreeView *treeview, GtkTreeIter *iter, GtkTreePath *path, gpointer callback_data)
+{
+ 	FMListView *view;
+ 	NautilusDirectory *directory;
+
+	view = FM_LIST_VIEW (callback_data);
+
+	if (fm_list_model_load_subdirectory (view->details->model, path, &directory)) {
+		fm_directory_view_add_subdirectory (FM_DIRECTORY_VIEW (view), directory);
+		
+		if (nautilus_directory_are_all_files_seen (directory)) {
+			fm_list_model_subdirectory_done_loading (view->details->model,
+								 directory);
+		} else {
+			g_signal_connect_object (directory, "done_loading",
+						 G_CALLBACK (subdirectory_done_loading_callback),
+						 view, 0);
+		}
+		
+		nautilus_directory_unref (directory);
+	}
+}
+
+struct UnloadDelayData {
+	NautilusFile *file;
+	FMListView *view;
+};
+
+static gboolean
+unload_file_timeout (gpointer data)
+{
+	struct UnloadDelayData *unload_data = data;
+	GtkTreeIter iter;
+	FMListModel *model;
+	GtkTreePath *path;
+
+	if (unload_data->view != NULL) {
+		model = unload_data->view->details->model;
+		if (fm_list_model_get_tree_iter_from_file (model,
+							   unload_data->file,
+							   &iter)) {
+			path = gtk_tree_model_get_path (GTK_TREE_MODEL (model), &iter);
+			if (!gtk_tree_view_row_expanded (unload_data->view->details->tree_view,
+							 path)) {
+				fm_list_model_unload_subdirectory (model, &iter);
+			}
+			gtk_tree_path_free (path);
+		}
+	}
+
+	eel_remove_weak_pointer (&unload_data->view);
+	
+	
+	g_object_unref (unload_data->file);
+	g_free (unload_data);
+	return FALSE;
+}
+
+static void
+row_collapsed_callback (GtkTreeView *treeview, GtkTreeIter *iter, GtkTreePath *path, gpointer callback_data)
+{
+ 	FMListView *view;
+ 	NautilusFile *file;
+	struct UnloadDelayData *unload_data;
+	
+	view = FM_LIST_VIEW (callback_data);
+		
+	gtk_tree_model_get (GTK_TREE_MODEL (view->details->model), 
+			    iter, 
+			    FM_LIST_MODEL_FILE_COLUMN, &file,
+			    -1);
+	
+	unload_data = g_new (struct UnloadDelayData, 1);
+	unload_data->view = view;
+	unload_data->file = file;
+
+	eel_add_weak_pointer (&unload_data->view);
+	
+	g_timeout_add (COLLAPSE_TO_UNLOAD_DELAY,
+		       unload_file_timeout,
+		       unload_data);
+}
+
+
+static void
+subdirectory_unloaded_callback (FMListModel *model,
+				NautilusDirectory *directory,
+				gpointer callback_data)
+{
+	FMListView *view;
+	
+	g_return_if_fail (FM_IS_LIST_MODEL (model));
+	g_return_if_fail (NAUTILUS_IS_DIRECTORY (directory));
+
+	view = FM_LIST_VIEW(callback_data);
+	
+	g_signal_handlers_disconnect_by_func (directory,
+					      G_CALLBACK (subdirectory_done_loading_callback),
+					      view);
+	fm_directory_view_remove_subdirectory (FM_DIRECTORY_VIEW (view), directory);
+}
+
 static gboolean
 key_press_callback (GtkWidget *widget, GdkEventKey *event, gpointer callback_data)
 {
@@ -1069,6 +1204,10 @@ create_and_set_up_tree_view (FMListView *view)
 				 G_CALLBACK (key_press_callback), view, 0);
 	g_signal_connect_object (view->details->tree_view, "popup_menu",
                                  G_CALLBACK (popup_menu_callback), view, 0);
+	g_signal_connect_object (view->details->tree_view, "row_expanded",
+                                 G_CALLBACK (row_expanded_callback), view, 0);
+	g_signal_connect_object (view->details->tree_view, "row_collapsed",
+                                 G_CALLBACK (row_collapsed_callback), view, 0);
 	
 	view->details->model = g_object_new (FM_TYPE_LIST_MODEL, NULL);
 	gtk_tree_view_set_model (view->details->tree_view, GTK_TREE_MODEL (view->details->model));
@@ -1078,6 +1217,9 @@ create_and_set_up_tree_view (FMListView *view)
 
 	g_signal_connect_object (view->details->model, "sort_column_changed",
 				 G_CALLBACK (sort_column_changed_callback), view, 0);
+	
+	g_signal_connect_object (view->details->model, "subdirectory_unloaded",
+				 G_CALLBACK (subdirectory_unloaded_callback), view, 0);
 
 	view->details->source_target_list = 
 		gtk_target_list_new (drag_types, num_drag_types);
@@ -1180,7 +1322,10 @@ create_and_set_up_tree_view (FMListView *view)
 static void
 fm_list_view_add_file (FMDirectoryView *view, NautilusFile *file)
 {
-	fm_list_model_add_file (FM_LIST_VIEW (view)->details->model, file);
+	FMListModel *model;
+	
+	model = FM_LIST_VIEW (view)->details->model;
+	fm_list_model_add_file (model, file);
 }
 
 static GList *
@@ -1374,8 +1519,10 @@ fm_list_view_get_selection_foreach_func (GtkTreeModel *model, GtkTreePath *path,
 			    FM_LIST_MODEL_FILE_COLUMN, &file,
 			    -1);
 
-	nautilus_file_ref (file);
-	(* list) = g_list_prepend ((* list), file);
+	if (file != NULL) {
+		nautilus_file_ref (file);
+		(* list) = g_list_prepend ((* list), file);
+	}
 }
 
 static GList *
@@ -1388,9 +1535,58 @@ fm_list_view_get_selection (FMDirectoryView *view)
 	gtk_tree_selection_selected_foreach (gtk_tree_view_get_selection (FM_LIST_VIEW (view)->details->tree_view),
 					     fm_list_view_get_selection_foreach_func, &list);
 
-	return list;
+	return g_list_reverse (list);
 }
 
+static void
+fm_list_view_get_selection_for_file_transfer_foreach_func (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data)
+{
+	NautilusFile *file;
+	struct SelectionForeachData *selection_data;
+	GtkTreeIter parent, child;
+
+	selection_data = data;
+
+	gtk_tree_model_get (model, iter,
+			    FM_LIST_MODEL_FILE_COLUMN, &file,
+			    -1);
+
+	if (file != NULL) {
+		/* If the parent folder is also selected, don't include this file in the
+		 * file operation, since that would copy it to the toplevel target instead
+		 * of keeping it as a child of the copied folder
+		 */
+		child = *iter;
+		while (gtk_tree_model_iter_parent (model, &parent, &child)) {
+			if (gtk_tree_selection_iter_is_selected (selection_data->selection,
+								 &parent)) {
+				return;
+			}
+			child = parent;
+		}
+		
+		nautilus_file_ref (file);
+		selection_data->list = g_list_prepend (selection_data->list, file);
+	}
+}
+
+
+static GList *
+fm_list_view_get_selection_for_file_transfer (FMDirectoryView *view)
+{
+	struct SelectionForeachData selection_data;
+
+	selection_data.list = NULL;
+	selection_data.selection = gtk_tree_view_get_selection (FM_LIST_VIEW (view)->details->tree_view);
+
+	gtk_tree_selection_selected_foreach (selection_data.selection,
+					     fm_list_view_get_selection_for_file_transfer_foreach_func, &selection_data);
+
+	return g_list_reverse (selection_data.list);
+}
+
+
+
 
 static guint
 fm_list_view_get_item_count (FMDirectoryView *view)
@@ -1465,7 +1661,7 @@ fm_list_view_remove_file (FMDirectoryView *view, NautilusFile *file)
 	   gtk_tree_path_free (file_path);
 		
 	   fm_list_model_remove_file (list_view->details->model, file);
-	   
+
 	   if (gtk_tree_row_reference_valid (row_reference)) {
 	      if (list_view->details->new_selection_path) {
 	          gtk_tree_path_free (list_view->details->new_selection_path);
@@ -2242,6 +2438,7 @@ fm_list_view_class_init (FMListViewClass *class)
 	fm_directory_view_class->file_changed = fm_list_view_file_changed;
 	fm_directory_view_class->get_background_widget = fm_list_view_get_background_widget;
 	fm_directory_view_class->get_selection = fm_list_view_get_selection;
+	fm_directory_view_class->get_selection_for_file_transfer = fm_list_view_get_selection_for_file_transfer;
 	fm_directory_view_class->get_item_count = fm_list_view_get_item_count;
 	fm_directory_view_class->is_empty = fm_list_view_is_empty;
 	fm_directory_view_class->remove_file = fm_list_view_remove_file;
