@@ -232,7 +232,9 @@ show_selection_idle_callback (gpointer callback_data)
 		(view->details->sort_model, path);
 	gtk_tree_path_free (path);
 	gtk_tree_view_set_cursor (view->details->tree_widget, sort_path, NULL, FALSE);
-	gtk_tree_view_scroll_to_cell (view->details->tree_widget, sort_path, NULL, FALSE, 0, 0);
+	if (GTK_WIDGET_REALIZED (view->details->tree_widget)) {
+		gtk_tree_view_scroll_to_cell (view->details->tree_widget, sort_path, NULL, FALSE, 0, 0);
+	}
 	gtk_tree_path_free (sort_path);
 
 	nautilus_file_unref (file);
@@ -927,15 +929,15 @@ convert_lines_to_str_list (char **lines, gboolean *cut)
 	int i;
 	GList *result;
 
+	*cut = FALSE;
+
 	if (lines[0] == NULL) {
 		return NULL;
 	}
 
 	if (strcmp (lines[0], "cut") == 0) {
 		*cut = TRUE;
-	} else if (strcmp (lines[0], "copy") == 0) {
-		*cut = FALSE;
-	} else {
+	} else if (strcmp (lines[0], "copy") != 0) {
 		return NULL;
 	}
 
@@ -954,7 +956,8 @@ paste_clipboard_data (FMTreeView *view,
 	char **lines;
 	gboolean cut;
 	GList *item_uris;
-	
+
+	cut = FALSE;
 	if (selection_data->type != copied_files_atom
 	    || selection_data->length <= 0) {
 		item_uris = NULL;
@@ -1482,6 +1485,12 @@ fm_tree_view_get_tab_label (NautilusSidebar *sidebar)
 	return g_strdup (_("Tree"));
 }
 
+static char *
+fm_tree_view_get_tab_tooltip (NautilusSidebar *sidebar)
+{
+	return g_strdup (_("Show Tree"));
+}
+
 static GdkPixbuf *
 fm_tree_view_get_tab_icon (NautilusSidebar *sidebar)
 {
@@ -1507,6 +1516,7 @@ fm_tree_view_iface_init (NautilusSidebarIface *iface)
 {
 	iface->get_sidebar_id = fm_tree_view_get_sidebar_id;
 	iface->get_tab_label = fm_tree_view_get_tab_label;
+	iface->get_tab_tooltip = fm_tree_view_get_tab_tooltip;
 	iface->get_tab_icon = fm_tree_view_get_tab_icon;
 	iface->is_visible_changed = fm_tree_view_is_visible_changed;
 }
