@@ -73,7 +73,6 @@
 #include <libnautilus-private/nautilus-mime-actions.h>
 #include <libnautilus-private/nautilus-program-choosing.h>
 #include <libnautilus-private/nautilus-sidebar.h>
-#include <libnautilus-private/nautilus-theme.h>
 #include <libnautilus-private/nautilus-view-factory.h>
 #include <libnautilus-private/nautilus-clipboard.h>
 #include <libnautilus-private/nautilus-undo.h>
@@ -94,6 +93,8 @@
 
 #define MAX_TITLE_LENGTH 180
 
+#define MENU_PATH_BOOKMARKS_PLACEHOLDER			"/MenuBar/Other Menus/Bookmarks/Bookmarks Placeholder"
+
 enum {
 	ARG_0,
 	ARG_APP_ID,
@@ -436,7 +437,6 @@ nautilus_navigation_window_finalize (GObject *object)
 	
 	window = NAUTILUS_NAVIGATION_WINDOW (object);
 
-	nautilus_navigation_window_remove_bookmarks_menu_callback (window);
 	nautilus_navigation_window_remove_go_menu_callback (window);
 	nautilus_navigation_window_clear_back_list (window);
 	nautilus_navigation_window_clear_forward_list (window);
@@ -456,6 +456,7 @@ nautilus_navigation_window_add_sidebar_panel (NautilusNavigationWindow *window,
 {
 	const char *sidebar_id;
 	char *label;
+	char *tooltip;
 	char *default_id;
 	GdkPixbuf *icon;
 
@@ -465,9 +466,12 @@ nautilus_navigation_window_add_sidebar_panel (NautilusNavigationWindow *window,
 	g_return_if_fail (g_list_find (window->sidebar_panels, sidebar_panel) == NULL);	
 
 	label = nautilus_sidebar_get_tab_label (sidebar_panel);
+	tooltip = nautilus_sidebar_get_tab_tooltip (sidebar_panel);
 	nautilus_side_pane_add_panel (window->sidebar, 
 				      GTK_WIDGET (sidebar_panel), 
-				      label);
+				      label,
+				      tooltip);
+	g_free (tooltip);
 	g_free (label);
 
 	icon = nautilus_sidebar_get_tab_icon (sidebar_panel);
@@ -605,14 +609,11 @@ create_view_as_menu_item (NautilusWindow *window,
 			  guint index)
 {
 	GtkWidget *menu_item;
-        char *menu_label;
 	const NautilusViewInfo *info;
 
 	info = nautilus_view_factory_lookup (identifier);
 
-	menu_label = g_strdup_printf (_("View as %s"), _(info->label));
-	menu_item = gtk_menu_item_new_with_mnemonic (menu_label);
-	g_free (menu_label);
+	menu_item = gtk_menu_item_new_with_mnemonic (_(info->view_as_label));
 
 	g_signal_connect_object (menu_item, "activate",
 				 G_CALLBACK (view_as_menu_switch_views_callback),
@@ -1094,7 +1095,8 @@ static void
 nautilus_navigation_window_class_init (NautilusNavigationWindowClass *class)
 {
 	NAUTILUS_WINDOW_CLASS (class)->window_type = NAUTILUS_WINDOW_NAVIGATION;
-
+	NAUTILUS_WINDOW_CLASS (class)->bookmarks_placeholder = MENU_PATH_BOOKMARKS_PLACEHOLDER;
+	
 	G_OBJECT_CLASS (class)->finalize = nautilus_navigation_window_finalize;
 	GTK_OBJECT_CLASS (class)->destroy = nautilus_navigation_window_destroy;
 	GTK_WIDGET_CLASS (class)->show = nautilus_navigation_window_show;
