@@ -36,6 +36,7 @@
 #include "nautilus-property-browser.h"
 #include "nautilus-signaller.h"
 #include "nautilus-window-manage-views.h"
+#include "nautilus-window-bookmarks.h"
 #include "nautilus-window-private.h"
 #include "nautilus-desktop-window.h"
 #include <eel/eel-debug.h>
@@ -138,6 +139,7 @@ void
 nautilus_menus_append_bookmark_to_menu (NautilusWindow *window, 
 					NautilusBookmark *bookmark, 
 					const char *parent_path,
+					const char *parent_id,
 					guint index_in_parent,
 					GtkActionGroup *action_group,
 					guint merge_id,
@@ -168,7 +170,7 @@ nautilus_menus_append_bookmark_to_menu (NautilusWindow *window,
 	/* Create menu item with pixbuf */
 	pixbuf = nautilus_bookmark_get_pixbuf (bookmark, NAUTILUS_ICON_SIZE_FOR_MENUS, FALSE);
 
-	action_name = g_strdup_printf ("bookmark_%d", index_in_parent);
+	action_name = g_strdup_printf ("bookmark_%s_%d", parent_id, index_in_parent);
 
 	action = gtk_action_new (action_name,
 				 display_name,
@@ -717,6 +719,8 @@ nautilus_window_initialize_menus (NautilusWindow *window)
 		action = gtk_action_group_get_action (action_group, NAUTILUS_ACTION_GO_TO_BURN_CD);
 		gtk_action_set_visible (action, FALSE);
 	}
+
+	nautilus_window_initialize_bookmarks_menu (window);
 }
 
 static GList *
@@ -808,3 +812,5 @@ nautilus_window_load_extension_menus (NautilusWindow *window)
 
 	g_list_free (items);
 }
+
+
