@@ -32,11 +32,13 @@
 
 #include "nautilus-actions.h"
 #include "nautilus-application.h"
+#include "nautilus-bookmark-list.h"
 #include "nautilus-bookmarks-window.h"
 #include "nautilus-information-panel.h"
 #include "nautilus-main.h"
 #include "nautilus-signaller.h"
 #include "nautilus-window-manage-views.h"
+#include "nautilus-window-bookmarks.h"
 #include "nautilus-zoom-control.h"
 #include <eel/eel-debug.h>
 #include <eel/eel-marshal.h>
@@ -535,6 +537,8 @@ nautilus_window_finalize (GObject *object)
 	
 	window = NAUTILUS_WINDOW (object);
 
+	nautilus_window_remove_bookmarks_menu_callback (window);
+	
 	nautilus_window_manage_views_finalize (window);
 
 	nautilus_window_set_viewed_file (window, NULL);
@@ -959,39 +963,6 @@ nautilus_window_display_error (NautilusWindow *window, const char *error_msg)
 	gtk_widget_show (dialog);
 }
 
-static char *
-compute_default_title (const char *text_uri)
-{
-	NautilusFile *file;
-	GnomeVFSURI *uri;
-	char *title, *displayname;
-	const char *hostname;
-
-	hostname = NULL;
-
-	if (text_uri) {
-		file = nautilus_file_get (text_uri);
-		uri = gnome_vfs_uri_new (text_uri);
-		if (uri && !gnome_vfs_uri_is_local (uri)) {
-			hostname = gnome_vfs_uri_get_host_name (uri);
-		}
-		displayname = nautilus_file_get_display_name (file);
-		if (hostname) {
-			title = g_strdup_printf (_("%s on %s"), displayname, hostname);
-			g_free (displayname);
-		} else {
-			title = displayname;
-		}
-		if (uri) {
-			gnome_vfs_uri_unref (uri);
-		}
-		nautilus_file_unref (file);
-	} else {
-		title = g_strdup ("");
-	}
-	
-	return title;
-}
 
 static char *
 real_get_title (NautilusWindow *window)
@@ -1007,7 +978,7 @@ real_get_title (NautilusWindow *window)
         }
         
 	if (title == NULL) {
-                title = compute_default_title (window->details->location);
+                title = nautilus_compute_title_for_uri (window->details->location);
         }
 
 	return title;
@@ -1415,6 +1386,12 @@ nautilus_window_set_hidden_files_mode (NautilusWindowInfo *window,
 			       mode);
 }
 
+static NautilusBookmarkList *
+nautilus_window_get_bookmark_list (NautilusWindow *window)
+{
+  return nautilus_get_bookmark_list ();
+}
+
 static void
 nautilus_window_info_iface_init (NautilusWindowInfoIface *iface)
 {
@@ -1429,6 +1406,7 @@ nautilus_window_info_iface_init (NautilusWindowInfoIface *iface)
 	iface->get_window_type = nautilus_window_get_window_type;
 	iface->get_title = nautilus_window_get_cached_title;
 	iface->get_history = nautilus_window_get_history;
+	iface->get_bookmark_list = nautilus_window_get_bookmark_list;
 	iface->get_current_location = nautilus_window_get_location;
 	iface->get_ui_manager = nautilus_window_get_ui_manager;
 	iface->get_selection_count = nautilus_window_get_selection_count;
@@ -1458,8 +1436,8 @@ nautilus_window_class_init (NautilusWindowClass *class)
 	g_object_class_install_property (G_OBJECT_CLASS (class),
 					 ARG_APP,
 					 g_param_spec_object ("app",
-							      _("Application"),
-							      _("The NautilusApplication associated with this window."),
+							      "Application",
+							      "The NautilusApplication associated with this window.",
 							      NAUTILUS_TYPE_APPLICATION,
 							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 	
